module Integrator
  class Devise::PasswordsController < DeviseController
    before_action :redirect_unless_user
    prepend_before_action :require_no_authentication
    append_before_action :assert_reset_token_passed, only: :edit

    def new
      self.resource = resource_class.new
    end

    def create
      self.resource = resource_class.send_reset_password_instructions(resource_params)
      yield resource if block_given?

      if successfully_sent?(resource)
        respond_with({}, location: login_path)
      else
        respond_with(resource)
      end
    end

    def edit
      self.resource = resource_class.new
      set_minimum_password_length
      resource.reset_password_token = params[:reset_password_token]
    end

    def update
      self.resource = resource_class.reset_password_by_token(resource_params)
      yield resource if block_given?
      if resource.errors.empty?
        resource.unlock_access! if unlockable?(resource)
        if ::Devise.sign_in_after_reset_password
          flash_message = resource.active_for_authentication? ? :updated : :updated_not_active
          set_flash_message!(:notice, flash_message)
          sign_in(resource_name, resource)
        else
          set_flash_message!(:notice, :updated_not_active)
        end
        respond_with resource, location: root_path
      else
        set_minimum_password_length
        respond_with resource
      end
    end

    protected

    def redirect_unless_user
      redirect_to signup_path unless Integrator::User.any?
    end

    def assert_reset_token_passed
      if params[:reset_password_token].blank?
        set_flash_message(:alert, :no_token)
        redirect_to login_path
      end
    end

    def unlockable?(resource)
      resource.respond_to?(:unlock_access!) &&
        resource.respond_to?(:unlock_strategy_enabled?) &&
        resource.unlock_strategy_enabled?(:email)
    end

    def translation_scope
      'devise.passwords'
    end
  end
end
