module Integrator
  class Devise::RegistrationsController < ::DeviseController
    before_action :redirect_if_has_user
    prepend_before_action :require_no_authentication, only: [ :new, :create ]
    prepend_before_action :set_minimum_password_length, only: [ :new ]

    def new
      build_resource({})
      yield resource if block_given?
      respond_with resource
    end

    def create
      build_resource(sign_up_params)
      resource.save
      yield resource if block_given?

      if resource.persisted?
        set_flash_message! :notice, :signed_up
        sign_up(resource_name, resource)
        respond_with resource, location: root_path
      else
        clean_up_passwords resource
        set_minimum_password_length
        respond_with resource
      end
    end

    protected

    def redirect_if_has_user
      redirect_to login_path if Integrator::User.any?
    end

    def build_resource(hash=nil)
      self.resource = resource_class.new_with_session(hash || {}, session)
    end

    def sign_up(resource_name, resource)
      sign_in(resource_name, resource)
    end

    def sign_up_params
      devise_parameter_sanitizer.sanitize(:sign_up)
    end

    def translation_scope
      'devise.registrations'
    end
  end
end
