module Integrator
  module Generators
    class InstallGenerator < Rails::Generators::Base
      include Rails::Generators::Migration

      desc 'Install Integrator CMS'

      def self.source_root
        @source_root ||= File.expand_path(File.join(File.dirname(__FILE__), 'templates'))
      end

      def copy_initializers
        @email = ask_email
        template 'devise.rb', 'config/initializers/devise.rb'
      end

      def insert_engine_routes
        route %(\n  # Integrator CMS\n  mount Integrator::Engine => '/admin', as: 'integrator'\n)
      end

      def create_migrations
        migration_template 'migrations/create_integrator_users.rb', 'db/migrate/create_integrator_users.rb'
      end

      def self.next_migration_number(dir)
        Time.now.utc.strftime('%Y%m%d%H%M%S')
      end

      private

      def generate_secret(count)
        SecureRandom.hex(count)
      end

      def ask_email
        email = ask('Enter your email for domain [no-reply@belweb.pro]:').downcase
        email.empty? ? 'no-reply@belweb.pro' : email
      end
    end
  end
end
